#include <fstream>
#include <iostream>
#include <list>
#include <string>

#include "arccli.h"

#include <arc/ftpcontrol.h>
#include <arc/joblist.h>
#include <arc/mdsquery.h>
#include <arc/notify.h>
#include <arc/resource.h>

#include <libintl.h>
#define _(A) dgettext("arccli", ((A)))


void arccat(const std::list<std::string>& jobs,
            const std::list<std::string>& clusterselect,
            const std::list<std::string>& clusterreject,
            const std::list<std::string>& status,
            const int whichfile,
            const int timeout,
            const bool anonymous) {

	std::list<std::string> jobids =
		GetJobIDsList(jobs, clusterselect, clusterreject);

	if (jobs.empty() && clusterselect.empty() && jobids.empty())
		throw ARCCLIError(_("No jobs"));

	if (jobids.empty())
		throw ARCCLIError(_("No valid jobnames/jobids given"));

	std::list<Job> joblist =
		GetJobInfo(jobids, MDS_FILTER_JOBINFO, anonymous, "", timeout);

	FTPControl ctrl;

	for (std::list<Job>::iterator jli = joblist.begin();
	     jli != joblist.end(); jli++) {

		if (jli->status.empty()) {
			notify(WARNING) << _("Job information not found")
			                << ": " << jli->id << std::endl;
			continue;
		}

		if (!status.empty() &&
		    find(status.begin(), status.end(), jli->status) == status.end())
			continue;

		if (whichfile == STDOUT || whichfile == STDERR) {
			if (jli->status == "DELETED") {
				notify(WARNING) << _("Job has already been deleted")
				                << ": " << jli->id << std::endl;
				continue;
			}

			if (jli->status == "ACCEPTING" || jli->status == "ACCEPTED" ||
			    jli->status == "PREPARING" || jli->status == "PREPARED" ||
			    jli->status == "INLRMS:Q") {
				notify(WARNING) << _("Job has not started yet")
				                << ": " << jli->id << std::endl;
				continue;
			}
		}

		if (whichfile == STDOUT && jli->stdout.empty()) {
			notify(ERROR) << _("Can not determine the stdout location")
			              << ": " << jli->id << std::endl;
			continue;
		}

		if (whichfile == STDERR && jli->stderr.empty()) {
			notify(ERROR) << _("Can not determine the stderr location")
			              << ": " << jli->id << std::endl;
			continue;
		}

		std::string url;
		if (whichfile == STDOUT)
			url = jli->id + '/' + jli->stdout;
		else if (whichfile == STDERR)
			url = jli->id + '/' + jli->stderr;
		else if (whichfile == GMLOG) {
			url = jli->id;
			url.insert(url.rfind('/'), "/info");
			url += "/errors";
		}

		std::string filename("/tmp/arccat.XXXXXX");
		if (mktemp((char*)filename.c_str()) == NULL)
			throw ARCCLIError(_("Could not create temporary file"));

		ctrl.Download(url, filename, timeout);

		std::ifstream is(filename.c_str());
		while(is)
			std::cout.put(is.get());
		is.close();

		unlink(filename.c_str());
	}
}
