#include <iostream>
#include <list>
#include <string>

#include "arccli.h"

#include <arc/alias.h>
#include <arc/certificate.h>
#include <arc/common.h>
#include <arc/configcore.h>
#include <arc/notify.h>
#include <arc/stringconv.h>

#include <getopt.h>
#include <libintl.h>
#define _(A) dgettext("arccli", ((A)))


static void arcusage(const std::string & progname, const char * optstring) {

	bool havegiisurlopt = false;
	bool havediropt = false;

	std::cout << _("Usage") << ": " << progname
	          << " [" << _("options") << "]";
	if (progname != "arcsync")
		if (progname == "arcsub")
			std::cout << " [" << _("filename") << " ...]";
		else
			std::cout << " [" << _("job") << " ...]";
	std::cout << std::endl << std::endl;
	std::cout << _("Options") << ":" << std::endl;
	for (int i = 0; optstring[i]; i++) {
		if (optstring[i] == ':') continue;
		if (optstring[i + 1] == ':') {
			switch (optstring[i]) {
			case 'c':
				std::cout << _(
"  -c, -cluster   [-]name       explicity select or reject a specific cluster"
                                       ) << std::endl;
				break;
			case 'C':
				std::cout << _(
"  -C, -clustlist [-]filename   list of clusters to select or reject"
                                       ) << std::endl;
				break;
			case 'd':
				std::cout << _(
"  -d, -debug     debuglevel    0 = FATAL, 1 = ERROR, 2 = WARNING,\n"
"                               3 = INFO,  4 = DEBUG, 5 = VERBOSE"
                                       ) << std::endl;
				break;
			case 'D':
				std::cout << _(
"      -dir                     download directory (the job directory will\n"
"                                 be created in this directory)"
                                       ) << std::endl;
				havediropt = true;
				break;
			case 'e':
				std::cout << _(
"  -e, -xrsl      xrslstring    xrslstring describing the job to be submitted"
                                       ) << std::endl;
				break;
			case 'f':
				std::cout << _(
"  -f, -file      filename      xrslfile describing the job to be submitted"
                                       ) << std::endl;
				break;
			case 'g':
				std::cout << _(
"  -g, -giisurl   url           url to a central GIIS"
                                       ) << std::endl;
				havegiisurlopt = true;
				break;
			case 'G':
				std::cout << _(
"  -G, -giislist  filename      list of GIIS urls"
                                       ) << std::endl;
				break;
			case 'i':
				std::cout << _(
"  -i, -joblist   filename      file containing a list of jobids"
                                       ) << std::endl;
				break;
			case 'k':
				std::cout << _(
"  -k, -kluster   [-]name       explicity select or reject a specific\n"
"                                 cluster as resubmission target"
                                       ) << std::endl;
				break;
			case 'K':
				std::cout << _(
"  -K, -klustlist [-]filename   list of clusters to select or reject as\n"
"                                 resubmission target"
                                       ) << std::endl;
				break;
			case 'o':
				std::cout << _(
"  -o, -joblist   filename      file where the jobids will be stored"
                                       ) << std::endl;
				break;
			case 's':
				std::cout << _(
"  -s, -status    statusstr     only select jobs whose status is statusstr"
                                       ) << std::endl;
				break;
			case 't':
				std::cout << _(
"  -t, -timeout   time          timeout in seconds (default 20)"
                                       ) << std::endl;
				break;
			}
		}
		else {
			switch (optstring[i]) {
			case 'a':
				std::cout << _(
"  -a, -all                     all jobs"
                                       ) << std::endl;
				break;
			case 'D':
				std::cout << _(
"      -dryrun                  add dryrun option to the xrsl"
                                       ) << std::endl;
				break;
			case 'e':
				std::cout << _(
"  -e, -stderr                  show stderr of the job"
                                       ) << std::endl;
				break;
			case 'f':
				if (progname == "arcclean") {
					std::cout << _(
"  -f, -force                   remove the job from the local list of jobs\n"
"                                 even if the job is not found in the MDS"
                                           ) << std::endl;
				}
				else {
					std::cout << _(
"  -f, -force                   do not ask for verification"
                                           ) << std::endl;
				}
				break;
			case 'h':
				std::cout << _(
"  -h, -help                    print this help"
                                       ) << std::endl;
				break;
			case 'j':
				std::cout << _(
"  -j, -usejobname              use the jobname instead of the short ID as\n"
"                                 the job directory name"
                                       ) << std::endl;
				break;
			case 'l':
				if (progname == "arccat") {
					std::cout << _(
"  -l, -gmlog                   show the grid manager's error log of the job"
                                           ) << std::endl;
				}
				else {
					std::cout << _(
"  -l, -long                    long format (more information)"
                                           ) << std::endl;
				}
				break;
			case 'o':
				std::cout << _(
"  -o, -stdout                  show stdout of the job (default)"
                                       ) << std::endl;
				break;
			case 'p':
				std::cout << _(
"  -p, -renewproxy              renew the job's proxy"
                                       ) << std::endl;
				break;
			case 'P':
				std::cout << _(
"      -dumpxrsl                do not submit - dump transformed xrsl to\n"
"                                 stdout"
                                       ) << std::endl;
				break;
			case 'q':
				std::cout << _(
"  -q, -queues                  show information about clusters and queues"
                                       ) << std::endl;
				break;
			case 'r':
				std::cout << _(
"      -keep                    keep files on gatekeeper (do not clean)"
                                       ) << std::endl;
				break;
			case 'v':
				std::cout << _(
"  -v, -version                 print version information"
                                       ) << std::endl;
				break;
			case 'x':
				std::cout << _(
"  -x, -anonymous               use anonymous bind for MDS queries (default)"
                                       ) << std::endl;
				break;
			case 'X':
				std::cout << _(
"  -X, -gsi                     use gsi-gssapi bind for MDS queries"
                                       ) << std::endl;
				break;
			}
		}
	}
	if (progname != "arcsync") {
		std::cout << std::endl;
		std::cout << _("Arguments") << ":" << std::endl;
		if (progname == "arcsub")
			std::cout << _(
"  filename ...                 xrslfiles describing the jobs to be submitted"
                                   ) << std::endl;
		else
			std::cout << _(
"  job ...                      list of jobids and/or jobnames"
                                   ) << std::endl;
	}
	if (havegiisurlopt) {
		std::cout << std::endl;
		std::cout << _(
"A url to a giis has the format ldap://hostname[:port]/basedn\n"
"If the port is omitted the default 2135 is used\n\n"
"If neither the -giisurl nor the -giislist option is given a list of urls is\n"
"read from the first existing source in the following list:\n\n"
"  1. giis options in [client] section in ${HOME}/.arc/conf\n"
"  2. giis options in [common] section in ${HOME}/.arc/conf\n"
"  3. ${HOME}/.nggiislist (one giis per line)\n"
"  4. giis options in [client] section in ${NORDUGRID_LOCATION}/etc/arc.conf\n"
"  5. giis options in [common] section in ${NORDUGRID_LOCATION}/etc/arc.conf\n"
"  6. ${NORDUGRID_LOCATION}/etc/giislist (one giis per line)\n"
"  7. giis options in [client] section in /etc/arc.conf\n"
"  8. giis options in [common] section in /etc/arc.conf\n"
"  9. /etc/giislist (one giis per line)"
                               ) << std::endl;
	}
	std::cout << std::endl;
	std::cout << _(
"A default value for the timeout can be set by the first existing of\n\n"
"  1. timeout option in [client] section in ${HOME}/.arc/conf\n"
"  2. timeout option in [common] section in ${HOME}/.arc/conf\n"
"  3. NGTIMEOUT option in ${HOME}/.ngrc (old format)\n"
"  4. timeout option in [client] section in ${NORDUGRID_LOCATION}/etc/arc.conf\n"
"  5. timeout option in [common] section in ${NORDUGRID_LOCATION}/etc/arc.conf\n"
"  6. timeout option in [client] section in /etc/arc.conf\n"
"  7. timeout option in [common] section in /etc/arc.conf"
                           ) << std::endl;
	std::cout << std::endl;
	std::cout << _(
"A default value for the debuglevel can be set by the first existing of\n\n"
"  1. debug option in [client] section in ${HOME}/.arc/conf\n"
"  2. debug option in [common] section in ${HOME}/.arc/conf\n"
"  3. NGDEBUG option in ${HOME}/.ngrc (old format)\n"
"  4. debug option in [client] section in ${NORDUGRID_LOCATION}/etc/arc.conf\n"
"  5. debug option in [common] section in ${NORDUGRID_LOCATION}/etc/arc.conf\n"
"  6. debug option in [client] section in /etc/arc.conf\n"
"  7. debug option in [common] section in /etc/arc.conf"
                           ) << std::endl;
	if (havediropt) {
		std::cout << std::endl;
		std::cout << _(
"A default value for the download directory other than the current working\n"
"directory can be set by the first existing of\n\n"
"  1. downloaddir option in [client] section in ${HOME}/.arc/conf\n"
"  2. downloaddir option in [common] section in ${HOME}/.arc/conf\n"
"  3. NGDOWNLOAD option in ${HOME}/.ngrc (old format)\n"
"  4. downloaddir option in [client] section in\n"
"       ${NORDUGRID_LOCATION}/etc/arc.conf\n"
"  5. downloaddir option in [common] section in\n"
"       ${NORDUGRID_LOCATION}/etc/arc.conf\n"
"  6. downloaddir option in [client] section in /etc/arc.conf\n"
"  7. downloaddir option in [common] section in /etc/arc.conf"
                               ) << std::endl;
	}
}


static std::string GetOption(const std::string& newname,
                             const std::string& oldname) {

	Config conf;
	std::string opt;

	std::string home = GetEnv("HOME");
	if (!home.empty()) {
		conf = ReadConfig(home + "/.arc/conf");
		opt = conf.FirstConfValue("client/" + newname);
		if (!opt.empty()) return opt;
		opt = conf.FirstConfValue("common/" + newname);
		if (!opt.empty()) return opt;

		// Old style configuration for backward compatibility
		conf = ReadConfig(home + "/.ngrc");
		opt = conf.FirstConfValue("UNDEFINED/" + oldname);
		if (!opt.empty()) return opt;
	}

	std::string nglocation = GetEnv("NORDUGRID_LOCATION");
	if (!nglocation.empty()) {
		conf = ReadConfig(nglocation + "/etc/arc.conf");
		opt = conf.FirstConfValue("client/" + newname);
		if (!opt.empty()) return opt;
		opt = conf.FirstConfValue("common/" + newname);
		if (!opt.empty()) return opt;
	}

	conf = ReadConfig(nglocation + "/etc/arc.conf");
	opt = conf.FirstConfValue("client/" + newname);
	if (!opt.empty()) return opt;
	opt = conf.FirstConfValue("common/" + newname);
	if (!opt.empty()) return opt;

	return std::string();
}


int main(int argc, char ** argv) {

	try {

		setlocale(LC_ALL, "");
		std::string nglocation = GetEnv("NORDUGRID_LOCATION");
		if (!nglocation.empty() && nglocation != "/usr") {
			bindtextdomain("arclib", (nglocation + "/share/locale").c_str());
			bindtextdomain("arccli", (nglocation + "/share/locale").c_str());
		}
		std::string debuglevel = GetOption("debug", "NGDEBUG");
		if (!debuglevel.empty())
			SetNotifyLevel((NotifyLevel) stringtoi(debuglevel));
		else
			SetNotifyLevel(WARNING);

		char * pos = strrchr(argv[0], '/');
		if (pos) argv[0] = pos + 1;
		std::string progname = argv[0];

		char * optstring;

		if (progname == "arccat")
			optstring = "ai:c:C:s:oelt:d:xXvh";
		else if (progname == "arcclean")
			optstring = "ai:c:C:s:ft:d:xXvh";
		else if (progname == "arcget")
			optstring = "ai:c:C:s:D:jrt:d:xXvh";
		else if (progname == "arckill")
			optstring = "ai:c:C:s:rt:d:xXvh";
		else if (progname == "arcrenew")
			optstring = "ai:c:C:s:t:d:xXvh";
		else if (progname == "arcresub")
			optstring = "ai:c:C:s:k:K:g:G:o:DPrt:d:xXvh";
		else if (progname == "arcresume")
			optstring = "ai:c:C:s:pt:d:xXvh";
		else if (progname == "arcstat")
			optstring = "ai:c:C:s:g:G:qlt:d:xXvh";
		else if (progname == "arcsub")
			optstring = "c:C:g:G:e:f:o:DPt:d:xXvh";
		else if (progname == "arcsync")
			optstring = "c:C:g:G:ft:d:xXvh";
		else
			throw ARCCLIError(
				_("Unknown program name") + (": " + progname));

		struct option longoptions[20];
		int nopt = 0;
		for (int i = 0; optstring[i]; i++) {
			if (optstring[i] == ':') continue;
			if (optstring[i + 1] == ':') {
				switch (optstring[i]) {
				case 'c':
					{
						struct option lopt = {"cluster", 1, 0, 'c'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'C':
					{
						struct option lopt = {"clustlist", 1, 0, 'C'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'd':
					{
						struct option lopt = {"debug", 1, 0, 'd'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'D':
					{
						struct option lopt = {"dir", 1, 0, 'D'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'e':
					{
						struct option lopt = {"xrsl", 1, 0, 'e'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'f':
					{
						struct option lopt = {"file", 1, 0, 'f'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'g':
					{
						struct option lopt = {"giisurl", 1, 0, 'g'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'G':
					{
						struct option lopt = {"giislist", 1, 0, 'G'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'i':
					{
						struct option lopt = {"joblist", 1, 0, 'i'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'k':
					{
						struct option lopt = {"kluster", 1, 0, 'c'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'K':
					{
						struct option lopt = {"klustlist", 1, 0, 'C'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'o':
					{
						struct option lopt = {"joblist", 1, 0, 'o'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 's':
					{
						struct option lopt = {"status", 1, 0, 's'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 't':
					{
						struct option lopt = {"timeout", 1, 0, 't'};
						longoptions[nopt++] = lopt;
						break;
					}
				}
			}
			else {
				switch (optstring[i]) {
				case 'a':
					{
						struct option lopt = {"all", 0, 0, 'a'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'D':
					{
						struct option lopt = {"dryrun", 0, 0, 'D'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'e':
					{
						struct option lopt = {"stderr", 0, 0, 'e'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'f':
					{
						struct option lopt = {"force", 0, 0, 'f'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'h':
					{
						struct option lopt = {"help", 0, 0, 'h'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'j':
					{
						struct option lopt = {"usejobname", 0, 0, 'j'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'l':
					{
						if (progname == "arccat") {
							struct option lopt = {"gmlog", 0, 0, 'l'};
							longoptions[nopt++] = lopt;
						}
						else {
							struct option lopt = {"long", 0, 0, 'l'};
							longoptions[nopt++] = lopt;
						}
						break;
					}
				case 'o':
					{
						struct option lopt = {"stdout", 0, 0, 'o'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'p':
					{
						struct option lopt = {"renewproxy", 0, 0, 'p'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'P':
					{
						struct option lopt = {"dumpxrsl", 0, 0, 'P'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'q':
					{
						struct option lopt = {"queues", 0, 0, 'q'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'r':
					{
						struct option lopt = {"keep", 0, 0, 'r'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'v':
					{
						struct option lopt = {"version", 0, 0, 'v'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'x':
					{
						struct option lopt = {"anonymous", 0, 0, 'x'};
						longoptions[nopt++] = lopt;
						break;
					}
				case 'X':
					{
						struct option lopt = {"gsi", 0, 0, 'X'};
						longoptions[nopt++] = lopt;
						break;
					}
				}
			}
		}

		struct option lopt = {0, 0, 0, 0};
		longoptions[nopt++] = lopt;

		// common options
		std::list<std::string> clusterreject;
		std::list<std::string> clusterselect;
		std::list<std::string> giisurls;
		bool all = false;
		std::list<std::string> status;
		int timeout;
		bool anonymous = true;
		// ngcat options
		int whichfile = STDOUT;
		// ngclean & ngsync options
		bool force = false;
		// ngget, ngkill & ngresub options
		bool keep = false;
		// ngget options
		std::string directory;
		bool usejobname = false;
		// ngstat options
		bool clusters = false;
		bool longlist = false;
		// ngsub options
		std::list<std::string> xrslstrings;
		// ngresub options
		std::list<std::string> klusterreject;
		std::list<std::string> klusterselect;
		// ngsub & ngresub options
		std::string joblistfile;
		bool dryrun = false;
		bool dumpxrsl = false;
		// ngresume options
		bool renewproxy = false;

		std::list<std::string> params;
		std::list<std::string> stringlist;

		std::string timeoutstr = GetOption("timeout", "NGTIMEOUT");
		if (!timeoutstr.empty())
			timeout = stringtoi(timeoutstr);
		else
			timeout = 20;

		int opt;
		while (opt != -1) {
			opt = getopt_long_only(argc, argv, optstring, longoptions, NULL);
			if (opt == -1) continue;
			if (optarg) {
				switch (opt) {
				case 'c':  // cluster
					if (optarg[0] == '-')
						clusterreject.push_back(&optarg[1]);
					else if (optarg[0] == '+')
						clusterselect.push_back(&optarg[1]);
					else
						clusterselect.push_back(optarg);
					break;
				case 'C':  // clustlist
					if (optarg[0] == '-') {
						stringlist = ReadFile(&optarg[1]);
						while (!stringlist.empty()) {
							clusterreject.push_back(*stringlist.begin());
							stringlist.pop_front();
						}
					}
					else if (optarg[0] == '+') {
						stringlist = ReadFile(&optarg[1]);
						while (!stringlist.empty()) {
							clusterselect.push_back(*stringlist.begin());
							stringlist.pop_front();
						}
					}
					else {
						stringlist = ReadFile(optarg);
						while (!stringlist.empty()) {
							clusterselect.push_back(*stringlist.begin());
							stringlist.pop_front();
						}
					}
					break;
				case 'd':  // debug
					SetNotifyLevel((NotifyLevel) atoi(optarg));
					break;
				case 'D':  // dir
					directory = optarg;
					break;
				case 'e':  // xrsl
					xrslstrings.push_back(optarg);
					break;
				case 'f':  // file
					params.push_back(optarg);
					break;
				case 'g':  // giisurl
					giisurls.push_back(optarg);
					break;
				case 'G':  // giislist
					stringlist = ReadFile(optarg);
					while (!stringlist.empty()) {
						giisurls.push_back(*stringlist.begin());
						stringlist.pop_front();
					}
					break;
				case 'i':  // joblist (in)
					stringlist = ReadFile(optarg);
					while (!stringlist.empty()) {
						params.push_back(*stringlist.begin());
						stringlist.pop_front();
					}
					break;
				case 'k':  // kluster
					if (optarg[0] == '-')
						klusterreject.push_back(&optarg[1]);
					else if (optarg[0] == '+')
						klusterselect.push_back(&optarg[1]);
					else
						klusterselect.push_back(optarg);
					break;
				case 'K':  // klustlist
					if (optarg[0] == '-') {
						stringlist = ReadFile(&optarg[1]);
						while (!stringlist.empty()) {
							klusterreject.push_back(*stringlist.begin());
							stringlist.pop_front();
						}
					}
					else if (optarg[0] == '+') {
						stringlist = ReadFile(&optarg[1]);
						while (!stringlist.empty()) {
							klusterselect.push_back(*stringlist.begin());
							stringlist.pop_front();
						}
					}
					else {
						stringlist = ReadFile(optarg);
						while (!stringlist.empty()) {
							klusterselect.push_back(*stringlist.begin());
							stringlist.pop_front();
						}
					}
					break;
				case 'o':  // joblist (out)
					joblistfile = optarg;
					break;
				case 's':  // status
					status.push_back(optarg);
					break;
				case 't':  // timeout
					timeout = atoi(optarg);
					break;
				default:
					throw ARCCLIError(_("Use -help option for help"));
					break;
				}
			}
			else {
				switch (opt) {
				case 'a':  // all
					all = true;
					break;
				case 'D':  // dryrun
					dryrun = true;
					break;
				case 'e':  // stderr
					whichfile = STDERR;
					break;
				case 'f':  // force
					force = true;
					break;
				case 'h':  // help
					arcusage(progname, optstring);
					return 0;
				case 'j':  // usejobname
					usejobname = true;
					break;
				case 'l':  // gmlog or long
					if (progname == "arccat")
						whichfile = GMLOG;
					else
						longlist = true;
					break;
				case 'o':  // stdout
					whichfile = STDOUT;
					break;
				case 'p':  // renewproxy
					renewproxy = true;
					break;
				case 'P':  // dumpxrsl
					dumpxrsl = true;
					break;
				case 'q':  // queues
					clusters = true;
					break;
				case 'r':  // keep
					keep = true;
					break;
				case 'v':  // version
					std::cout << progname << ": " << _("version") << " "
							  << VERSION << std::endl;
					return 0;
				case 'x':  // anonymous
					anonymous = true;
					break;
				case 'X':  // gsi
					anonymous = false;
					break;
				default:
					throw ARCCLIError(_("Use -help option for help"));
					break;
				}
			}
		}

		while (argc > optind) params.push_back(argv[optind++]);

		if (progname == "arcsub" && params.empty() && xrslstrings.empty()) {
			std::string xrsl;
			std::string line;
			while (getline(std::cin, line) && line != ".") xrsl += line;
			xrslstrings.push_back(xrsl);
		}

		ResolveAliases(clusterselect);
		ResolveAliases(clusterreject);
		ResolveAliases(klusterselect);
		ResolveAliases(klusterreject);

		if (progname == "arcget") {
			if (directory.empty())
				directory = GetOption("downloaddir", "NGDOWNLOAD");
			if (directory.empty()) {
				char buffer[PATH_MAX];
				getcwd (buffer, PATH_MAX);
				directory = buffer;
			}
		}

		if (progname != "arcsub" && progname != "arcsync") {
			if (clusters) {
				if (all || !params.empty() || !status.empty())
					throw ARCCLIError(_("Incompatible Options"));
			}
			else {
				if (all && !params.empty() && !clusterselect.empty() &&
				    !clusterreject.empty() && !status.empty())
					throw ARCCLIError(_("Incompatible Options"));

				if (!all && params.empty() && clusterselect.empty() &&
				    clusterreject.empty() && status.empty())
					throw ARCCLIError(_("Incompatible Options"));
			}
		}

		Certificate proxy(PROXY);

		notify(WARNING) << _("Proxy subject name") << ": "
		                << proxy.GetSN() << std::endl;
		notify(WARNING) << _("Proxy valid to") << ": "
		                << proxy.ExpiryTime() << std::endl;

		if (proxy.IsExpired())
			throw ARCLibError(_("The proxy has expired"));

		notify(WARNING) << _("Proxy valid for") << ": "
		                << proxy.ValidFor() << std::endl;

		if (progname == "arccat")
			arccat(params, clusterselect, clusterreject, status, whichfile,
			       timeout, anonymous);
		else if (progname == "arcclean")
			arcclean(params, clusterselect, clusterreject, status, force,
			         timeout, anonymous);
		else if (progname == "arcget")
			arcget(params, clusterselect, clusterreject, status,
			       directory, usejobname, keep, timeout, anonymous);
		else if (progname == "arckill")
			arckill(params, clusterselect, clusterreject, status, keep,
			        timeout, anonymous);
		else if (progname == "arcrenew")
			arcrenew(params, clusterselect, clusterreject, status,
			         timeout, anonymous);
		else if (progname == "arcresub")
			arcresub(params, clusterselect, clusterreject, status,
			         klusterselect, klusterreject, giisurls, joblistfile,
			         dryrun, dumpxrsl, keep, timeout, anonymous);
		else if (progname == "arcresume")
			arcresume(params, clusterselect, clusterreject, status,
			          renewproxy, timeout, anonymous);
		else if (progname == "arcstat")
			arcstat(params, clusterselect, clusterreject, status,
			        giisurls, clusters, longlist, timeout, anonymous);
		else if (progname == "arcsub")
			arcsub(params, xrslstrings, clusterselect, clusterreject, giisurls,
			       joblistfile, dryrun, dumpxrsl, timeout, anonymous);
		else if (progname == "arcsync")
			arcsync(clusterselect, clusterreject, giisurls, force, timeout,
			        anonymous);
	}

	catch(ARCLibError e) {

		notify(ERROR) << e.what() << std::endl;
		return 1;
	}

	return 0;
}
