#include "../std.h"

#include <string>
#include <list>

#include <fstream>
#include "info_files.h"
#include <arc/stringconv.h>
#include <arc/job_jsdl.h>
#include <arc/job_xrsl.h>
#include <arc/certificate.h>
#include "../config/conf.h"

#include "info_log.h"

const char * const sfx_local       = ".local";
const char * const sfx_rsl         = ".description";
const char * const sfx_diag        = ".diag";

// Create multiple files for sending to logger
// TODO - make it SOAP XML so that they could be sent directly
bool job_log_make_file(const JobDescription &desc,JobUser &user,const std::string &url) {
  std::string fname_dst = user.ControlDir()+"/logs/"+desc.get_id()+".XXXXXX";
  std::string fname_src;
  std::string status;
  int h_dst;
  int l;
  time_t t;
  char buf[256];
  if((h_dst=mkstemp((char*)(fname_dst.c_str()))) == -1) {
    return false;
  };
  chmod(fname_dst.c_str(),S_IRUSR | S_IWUSR);
  fix_file_owner(fname_dst,desc,user);
  fix_file_permissions(fname_dst,false);
#if defined __GNUC__ && __GNUC__ >= 3
  std::ofstream o_dst(fname_dst.c_str());
  close(h_dst);
#else
  std::ofstream o_dst(h_dst);
#endif
  // URL to send info to
  if(url.length()) {
    o_dst<<"loggerurl="<<url<<std::endl; if(o_dst.fail()) goto error;
  };
  // Copy job description
  {
  fname_src = user.ControlDir() + "/job." + desc.get_id() + sfx_rsl;
  int h_src=open(fname_src.c_str(),O_RDONLY);
  if(h_src==-1) goto error; 
  o_dst<<"description=";
  for(;;) {
    l=read(h_src,buf,sizeof(buf));
    if(l==0) break;
    if(l==-1) goto error;
    for(char* p=buf;p;) { p=(char*)memchr(buf,'\r',l); if(p) (*p)=' '; };
    for(char* p=buf;p;) { p=(char*)memchr(buf,'\n',l); if(p) (*p)=' '; };
    o_dst.write(buf,l);
    if(o_dst.fail()) goto error;
  };
  o_dst<<std::endl;
  struct stat st;
  if(fstat(h_src,&st) == 0) {
    struct passwd pw_;
    struct passwd *pw;
    char buf[BUFSIZ];
    getpwuid_r(st.st_uid,&pw_,buf,BUFSIZ,&pw);
    if(pw != NULL) {
      if(pw->pw_name) o_dst<<"localuser="<<pw->pw_name<<std::endl;
    };
  };
  close(h_src);
  };
  // Start time and identifier
  t = job_mark_time(fname_src);
  o_dst<<"submissiontime="<<mds_time(t)<<std::endl;
  o_dst<<"ngjobid="<<desc.get_id()<<std::endl;
  if(o_dst.fail()) goto error;
  // Analyze job.ID.local and store relevant information
  {
  fname_src = user.ControlDir() + "/job." + desc.get_id() + sfx_local;
  std::ifstream i_src(fname_src.c_str());
  for(;;) {
    if(i_src.fail()) goto error;
    if(o_dst.fail()) goto error;
    if(i_src.eof()) break;
    std::string value;
    std::string key = config_read_line(i_src,value,'=');
    if(key=="subject") { o_dst<<"usersn="<<value<<std::endl; }
    else if(key=="lrms") { o_dst<<"lrms="<<value<<std::endl; }
    else if(key=="queue") { o_dst<<"queue="<<value<<std::endl; }
    else if(key=="localid") { o_dst<<"localjobid="<<value<<std::endl; }
    else if(key=="jobname") { o_dst<<"jobname="<<value<<std::endl; }
    else if(key=="clientname") { o_dst<<"clienthost="<<value<<std::endl; }
  };
  };
  // Extract requested resources
  {
  fname_src = user.ControlDir() + "/job." + desc.get_id() + sfx_rsl;
  std::ifstream i_src(fname_src.c_str());
  JobRequest* job = NULL;
  try {
    job = new JobRequestJSDL(i_src); 
  } catch (std::exception e) {
    try {
      job = new JobRequestXRSL(i_src); 
    } catch (std::exception e) {
      goto error;
    };
  };
  if(job->Memory()) o_dst<<"requestedmemory="<<job->Memory()<<std::endl;
  if(job->CPUTime()) o_dst<<"requestedcputime="<<job->CPUTime()<<std::endl;
  //if(job->WallTime()) o_dst<<"requestedwalltime="<<job->WallTime()<<std::endl;
  delete job;
  };
  // Analyze diagnostics and store relevant information
  {
  fname_src = user.ControlDir() + "/job." + desc.get_id() + sfx_diag;
  std::ifstream i_src(fname_src.c_str());
  if(!i_src.fail()) {
    std::string nodenames;
    int nodecount = 0;
    int cputime = 0;
    for(;;) {
      if(i_src.fail()) goto error;
      if(o_dst.fail()) goto error;
      if(i_src.eof()) break;
      std::string value;
      std::string key = config_read_line(i_src,value,'=');
      if(key=="nodename") {
        if(nodecount) nodenames+=":"; nodenames+=value;
        nodecount++;
      } else if(strcasecmp(key.c_str(),"walltime") == 0) {
        o_dst<<"usedwalltime="<<value<<std::endl;
      } else if(strcasecmp(key.c_str(),"kerneltime") == 0) {
        try { cputime+=stringtoui(value); } catch (std::exception e) { };
      } else if(strcasecmp(key.c_str(),"usertime") == 0) {
        try { cputime+=stringtoui(value); } catch (std::exception e) { };
      } else if(strcasecmp(key.c_str(),"averagetotalmemory") == 0) {
        o_dst<<"usedmemory="<<value<<std::endl;
      } else if(strcasecmp(key.c_str(),"exitcode") == 0) {
        o_dst<<"exitcode="<<value<<std::endl;
      };
    };
    if(nodecount) {
      o_dst<<"nodename="<<nodenames<<std::endl;
      o_dst<<"nodecount="<<nodecount<<std::endl;
    };
    o_dst<<"usedcputime="<<cputime<<std::endl;
  };
  };
  // Endtime and failure reason
  if(desc.get_state() == JOB_STATE_FINISHED) {
    status="completed";
    t = job_state_time(desc.get_id(),user);
    if(t == 0) t=::time(NULL);
    o_dst<<"endtime="<<mds_time(t)<<std::endl;
    if(job_failed_mark_check(desc.get_id(),user)) {
      std::string failure = job_failed_mark_read(desc.get_id(),user);
      o_dst<<"failurestring="<<failure<<std::endl;
      status="failed";
    };
  };
  if(status.length()) o_dst<<"status="<<status<<std::endl;
  // Identity of cluster
  try {
    Certificate cert(HOSTCERT);
    o_dst<<"cluster="<<cert.GetSN()<<std::endl;
  } catch (std::exception e) { };
  if(o_dst.fail()) goto error;
  o_dst.close();
  return true;
error:
  o_dst.close();
  unlink(fname_dst.c_str());
  return false;
}

