#ifndef SERVICE_H
#define SERVICE_H

#include "common.h"
#include "pointers.h"
#include "backend_file.h"

#include "message.h"

/// Interface to remote service
class FlOServiceRemote: public SafeObject {
 private:
  /// URL to contact service
  std::string url_;
  /// DN of certificate used to identfy service
  std::string id_;
  /// Queue of messages to be sent to this service
  std::list<FlOMessage> msgs_;
  LockSimple msgs_lock_;
  /// Queue of registrations to be sent to this service
  std::list<FlORegistration> regs_;
  LockSimple regs_lock_;
 public:
  FlOServiceRemote(const std::string& url,const std::string& id);
  virtual ~FlOServiceRemote(void);
  const std::string& url(void) { return url_; };
  const std::string& id(void) { return id_; };
  /// Put messages into a queue (could also send it immediately)
  void notify(const FlOMessage& m);
  /// Register s to this service
  void registr(FlOService& s);
  /// Called by parent service to run internal tasks periodically
  void maintain(void);
};

/// Kind of main class. Represents local service itself.
class FlOService {
 private:
  bool valid;
  /// URL of interface
  std::string url_;
  /// List or registered services
  std::list<pFlOServiceRemote> services;
  /// List of handled objects
  std::list<pFlOObject> objects;
  /// Interface to backend which provides permanent storage
  FlOStorageFile storage;
 public:
  FlOService(const std::string& url,const std::string& path);
  operator bool(void) const { return valid; };
  bool operator!(void) const { return !valid; };
  const std::string& url(void) const { return url_; };
  /// Add new object to be handled
  void add(pFlOObject o);
  /// Add new service to be notified about new objects
  void add(pFlOServiceRemote s);
  /// Inform registered services about changes (calls corresponding 
  /// methods of hadled objects)
  void notify(const FlOMessage& m);
  /// Process recieved notification
  void notification(FlOMessage& msg,pFlOServiceRemote& srv);
  /// Find service in list by id or create a new one
  pFlOServiceRemote service(const std::string& id);
  /// Register itself service to remote service
  void registr(pFlOServiceRemote s);
  /// Find object by id (NULL if not found)
  pFlOObject find(std::string id);
  /// Called to run internal tasks periodically
  void maintain(void);
};

#endif // SERVICE_H

